<?php

/*
+---------------------------------------------------------------------------+
| Max Media Manager v0.3                                                    |
| =================                                                         |
|                                                                           |
| Copyright (c) 2003-2006 m3 Media Services Limited                         |
| For contact details, see: http://www.m3.net/                              |
|                                                                           |
| This program is free software; you can redistribute it and/or modify      |
| it under the terms of the GNU General Public License as published by      |
| the Free Software Foundation; either version 2 of the License, or         |
| (at your option) any later version.                                       |
|                                                                           |
| This program is distributed in the hope that it will be useful,           |
| but WITHOUT ANY WARRANTY; without even the implied warranty of            |
| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
| GNU General Public License for more details.                              |
|                                                                           |
| You should have received a copy of the GNU General Public License         |
| along with this program; if not, write to the Free Software               |
| Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA |
+---------------------------------------------------------------------------+
$Id: StatsController.php 4640 2006-04-07 09:42:19Z matteo@beccati.com $
*/

require_once MAX_PATH . '/lib/max/Dal/Admin.php';
require_once MAX_PATH . '/www/admin/lib-statistics.inc.php';
require_once MAX_PATH . '/lib/max/other/common.php';
require_once MAX_PATH . '/lib/max/Admin_DA.php';
require_once MAX_PATH . '/lib/max/other/html.php';
require_once MAX_PATH . '/lib/max/other/stats.php';
require_once 'HTML/Template/Flexy.php';
require_once 'HTML/Template/Flexy/Element.php';

/**
 * Abstract controller class for displaying statistics screens
 *
 * Always use the factory method to instantiate fields -- it will create
 * the right subclass for you.
 *
 * @package    Max
 * @subpackage Admin_Statistics
 * @author     Matteo Beccati <matteo@beccati.com>
 * @abstract
 *
 * @see StatsControllerFactory
 */
class StatsController
{
    /**#@+
     * @var string
     */
    var $template;
    var $title;
    /**#@-*/
    
    /** @var object */
    var $dateRangeSelector;
    
    /**#@+
     * @var string
     */
    var $pageName;
    var $pageURI;
    var $pageId;
    /**#@-*/
    
    /**#@+
     * @var array
     */
    var $pageParams;
    var $pageSections;
    var $pagePrefs;
    var $pageContext;
    var $pageShortcuts;
    var $globalPrefs;
    /**#@-*/
    
    /** @var integer */
    var $tabindex;
    
    /**#@+
     * @var boolean
     */
    var $noStatsAvailable = false;
    var $showDateRangeSelector = false;
    var $showTotals = false;
    var $showAverage = false;
    /**#@-*/
    
    /** @var array */
    var $showColumns;
    
    /**#@+
     * @var array
     */
    var $aDates;
    var $total;
    var $average;
    /**#@-*/
    
    /**#@+
     * @var string
     */
    var $averageSpan;
    var $listOrder;
    var $listOrderDirection;
    /**#@-*/
    
    /** @var string */
    var $phpAds_TextDirection;


    /**
     * PHP5-style constructor
     */
    function __constructor() 
    {
        $this->pageName = basename($_SERVER['PHP_SELF']);
        $this->pagePrefs = array();
        $this->pageParams = array();
        $this->pageShortcuts = array();
        $this->globalPrefs = array();

        $this->phpAds_TextDirection = $GLOBALS['phpAds_TextDirection'];
        
        $this->totals = array(
            'sum_requests'    => 0,
            'sum_views'       => 0,
            'sum_clicks'      => 0,
            'sum_conversions' => 0,
            'sum_ctr'         => 0,
            'sum_sr'          => 0
        );
        
        $this->showColumns = array(
            'requests'    => 1,
            'views'       => 1,
            'clicks'      => 1,
            'conversions' => 1,
            'ctr'         => 1,
            'sr'          => 1
        );

        $this->start();
    }
    
    /**
     * PHP4-style constructor
     */
    function StatsController() 
    {
        $this->__constructor();
    }

    /**
     * Abstract function which should contain the page data generation
     */
    function start() 
    {
        MAX::raiseError('Programmer failure: "start" is an abstract method, and must be over-riden.', MAX_ERROR_NOMETHOD);
    }
    
    /**
     * Return a translated string
     *
     * @param string Name of the translation string
     */
    function getTranslation($str) 
    {
        if (preg_match('/^(str|key)/', $str) && isset($GLOBALS[$str])) {
            $str = $GLOBALS[$str];
        }
        
        return $str;
    }

    /**
     * Return a translated string -- helper function for Flexy
     *
     * @param string Name of the translation string
     *
     * @see getTranslation
     */
    function tr($str)
    {
        return StatsController::getTranslation($str);
    }

    /**
     * Initialize the date-range selector
     *
     * You need to call this function for every page which should diplay
     * the date-range selector
     */
    function initDateRangeSelector()
    {
        require_once MAX_PATH . '/lib/max/Admin/UI/FieldFactory.php';
        
        $aPeriod = array();
        $aPeriod['period_preset'] = MAX_getStoredValue('period_preset', 'today');
        $aPeriod['period_start']  = MAX_getStoredValue('period_start', date('Y-m-d'));
        $aPeriod['period_end']    = MAX_getStoredValue('period_end', date('Y-m-d'));
                    
        $this->dateRangeSelector = &FieldFactory::newField('date-range');
        $this->dateRangeSelector->_name = 'period';
        $this->dateRangeSelector->_autoSubmit = true;
        $this->dateRangeSelector->setValueFromArray($aPeriod);
        
        $this->aDates = array(
            'day_begin' => $this->dateRangeSelector->getStartDate(),
            'day_end'   => $this->dateRangeSelector->getEndDate(),
        );
        
        if (!is_null($this->aDates)) {
            $this->aDates['day_begin'] = $this->aDates['day_begin']->format('%Y-%m-%d');
            $this->aDates['day_end']   = $this->aDates['day_end']->format('%Y-%m-%d');
        } else {
            $aDates = array();
        }
        
        $this->globalPrefs['period_preset'] = $this->dateRangeSelector->_fieldSelectionValue;
        $this->globalPrefs['period_start']  = $this->aDates['day_begin'];
        $this->globalPrefs['period_end']    = $this->aDates['day_end'];
        
        $this->showDateRangeSelector = true;
    }
    
    /**
     * Output the date-range selector -- helper function for Flexy
     *
     * This function is meant to be called within a template
     */
    function showDateRangeSelector()
    {
        $this->dateRangeSelector->_tabIndex = $this->tabindex;

        echo "
        <form id='period_form' name='period_form' action='{$this->pageName}'>";
        
        _displayHiddenValues($this->pageParams);

        $this->dateRangeSelector->display();
        
        echo "
        <a href='#' onclick='return periodFormSubmit()'>
        <img src='images/{$GLOBALS['phpAds_TextDirection']}/go_blue.gif' border='0'></a>
        </form>";

        $this->tabindex = $this->dateRangeSelector->_tabIndex;
    }
    
    /**
     * Format a row of stats according to the user preferences
     *
     * @static
     *
     * @param array Row of stats
     */
    function formatStats(&$row)
    {
        $row['sum_requests'] = phpAds_formatNumber($row['sum_requests']);
        $row['sum_views'] = phpAds_formatNumber($row['sum_views']);
        $row['sum_clicks'] = phpAds_formatNumber($row['sum_clicks']);
        $row['sum_conversions'] = phpAds_formatNumber($row['sum_conversions']);

        $row['sum_ctr'] = phpAds_formatPercentage($row['sum_ctr']);
        $row['sum_sr'] = phpAds_formatPercentage($row['sum_sr']);
    }

    /**
     * Format a row of stats recursively according to the user preferences
     *
     * This function is useful when formatting stats for entities which also
     * have subentities
     *
     * @static
     *
     * @param array Row of stats
     */
    function formatStatsRecursive(&$row)
    {
        $this->formatStats($row);
        
        if (isset($row['subentities']) && is_array($row['subentities'])) {
            foreach (array_keys($row['subentities']) as $key) {
                $this->formatStatsRecursive($row['subentities'][$key]);
            }
        }
    }
    
    /**
     * Generate CTR and SR ratios
     *
     * @static
     *
     * @param array Row of stats
     */
    function summarizeStats(&$row)
    {
        $row['sum_ctr'] = phpAds_buildRatio($row['sum_clicks'], $row['sum_views']);
        $row['sum_sr'] = phpAds_buildRatio($row['sum_conversions'], $row['sum_clicks']);
    }

    /**
     * Calculate average requests, impressions, clicks and conversions
     *
     * @static
     *
     * @param array Total stats
     * @param array Number of entries
     */
    function summarizeAverage($total, $count)
    {
        if ($count < 2) {
            return false;
        }
    
        $average = array();
        $average['sum_requests']    = $total['sum_requests'] / $count;
        $average['sum_views']       = $total['sum_views'] / $count;
        $average['sum_clicks']      = $total['sum_clicks'] / $count;
        $average['sum_conversions'] = $total['sum_conversions'] / $count;

        $this->formatStats($average);
        
        return $average;
    }

    /**
     * Calculate total requests, impressions, clicks and conversions
     *
     * @param array Rows of stats
     * @param boolean Calculate average data too
     */
    function summarizeTotals(&$rows, $average = false)
    {
        foreach ($rows as $row) {
            $this->total['sum_requests'] += $row['sum_requests'];
            $this->total['sum_views'] += $row['sum_views'];
            $this->total['sum_clicks'] += $row['sum_clicks'];
            $this->total['sum_conversions'] += $row['sum_conversions'];
        }
        
        $this->total['sum_ctr'] = phpAds_buildRatio($this->total['sum_clicks'], $this->total['sum_views']);
        $this->total['sum_sr'] = phpAds_buildRatio($this->total['sum_conversions'], $this->total['sum_clicks']);
        
        if ($average) {
            $this->average = $this->summarizeAverage($this->total, count($rows));
            $this->averageSpan = count($rows);
            $this->showAverage = $this->average !== false;
        }
        
        $this->noStatsAvailable = !$this->total['sum_requests'] && !$this->total['sum_views'] &&
                                  !$this->total['sum_clicks'] && !$this->total['sum_conversions'];
        
        $this->formatStats($this->total);

        foreach (array_keys($rows) as $k) {
            $this->formatStatsRecursive($rows[$k]);
        }
        
        $this->showTotals = true;
    }
        
    /**
     * Return the link to change sorting field and order -- helper function for Flexy
     *
     * @param string Name of the field
     * @param boolean Reverse default sorting
     * @return string The href parameter
     */
    function listOrderHref($fieldname, $reverse = false)
    {
        if ($this->listOrderField == $fieldname) {
            $orderdirection = $this->listOrderDirection == 'up' ? 'down' : 'up';
        } else {
            $orderdirection = $reverse ? 'down' : 'up';
        }
        
        return "{$this->pageURI}listorder={$fieldname}&orderdirection={$orderdirection}";
    }

    /**
     * Return the link to change sorting field and order using reversed
     * order by default -- helper function for Flexy
     *
     * @param string Name of the field
     * @return string The href parameter
     */
    function listOrderHrefRev($fieldname)
    {
        return $this->listOrderHref($fieldname, true);
    }
    
    /**
     * Return the image to show current ordering -- helper function for Flexy
     *
     * The function returns false if the data is not sorted by $fieldname
     *
     * @param string Name of the field
     * @return mixed Image src parameter  
     */
    function listOrderImage($fieldname)
    {
        if ($this->listOrderField == $fieldname) {
            return "images/caret-".($this->listOrderDirection == 'up' ? 'u': 'ds').".gif";
        } else {
            return false;
        }
    }
    
    /**
     * Add page paramters to a page name and a terminating ? or & character
     *
     * @static
     *
     * @param string Page name
     * @return string URI
     */
    function uriAddParams($pageName)
    {
        $pageURI = $pageName.'?';
        foreach ($this->pageParams as $k => $v) {
            $pageURI .= urlencode($k).'='.urlencode($v).'&';
        }
        
        return $pageURI;
    }
    
    /**
     * Generate the current page URI with the correct page parameters
     * and store it in {@link pageURI} for use within templates
     */
    function generatePageURI()
    {
        $this->pageURI = $this->uriAddParams($this->pageName);
    }
    
    /**
     * Save the preferences previously assigned to {@link pagePrefs} and {@link globalPrefs}
     */
    function savePrefs()
    {
        foreach ($this->pagePrefs as $k => $v)
            $GLOBALS['session']['prefs'][$this->pageName][$k] = $v;
        foreach ($this->globalPrefs as $k => $v)
            $GLOBALS['session']['prefs']['GLOBALS'][$k] = $v;
        
        phpAds_sessionDataStore();
    }

    /**
     * Output context for the left navigation bar
     *
     * Note: the function uses phpAds_PageContext(), it doesn't make any output itself
     */
    function showContext($type, $current_id = 0)
    {
        $aParams = array();
        
        switch ($type) {
        
        case 'advertisers':
            if (phpAds_isUser(phpAds_Admin) || phpAds_isUser(phpAds_Agency)) {
                if (phpAds_isUser(phpAds_Agency)) {
                    $aParams['agency_id'] = phpAds_getUserID();
                }
                
                $advertisers = Admin_DA::getAdvertisers($aParams, false);
                foreach ($advertisers as $advertiser) {
                    phpAds_PageContext (
                        phpAds_buildName($advertiser['advertiser_id'], $advertiser['name']),
                        $this->pageName.'?clientid='.$advertiser['advertiser_id'],
                        $current_id == $advertiser['advertiser_id']
                    );
                }
            }
            break;
            
        case 'campaigns':
            $aParams['advertiser_id'] = $this->pageParams['clientid'];
            
            $campaigns = Admin_DA::getPlacements($aParams, false);
            foreach ($campaigns as $campaign) {
                phpAds_PageContext (
                    phpAds_buildName($campaign['placement_id'], $campaign['name']),
                    $this->pageName.'?clientid='.$this->pageParams['clientid'].'&campaignid='.$campaign['placement_id'],
                    $current_id == $campaign['placement_id']
                );
            }
            break;
            
        case 'banners':
            $aParams['placement_id'] = $this->pageParams['campaignid'];
            
            $ads = Admin_DA::getAds($aParams, false);
            foreach ($banners as $banner) {
                phpAds_PageContext (
                    phpAds_buildName($banner['ad_id'], $banner['name']),
                    $this->pageName.'?clientid='.$this->pageParams['clientid'].'&campaignid='.$this->pageParams['campaignid'].'&bannerid='. $banner['ad_id'],
                    $current_id == $banner['ad_id']
                );
            }
            break;
            
        case 'publishers':
            if (phpAds_isUser(phpAds_Admin) || phpAds_isUser(phpAds_Agency)) {
                if (phpAds_isUser(phpAds_Agency)) {
                    $aParams['agency_id'] = phpAds_getUserID();
                }
                
                $publishers = Admin_DA::getPublishers($aParams, false);
                foreach ($publishers as $publisher) {
                    phpAds_PageContext (
                        phpAds_buildName($publisher['publisher_id'], $publisher['name']),
                        $this->pageName.'?affiliateid='.$publisher['publisher_id'],
                        $current_id == $publisher['publisher_id']
                    );
                }
            }
            break;
            
        case 'zones':
            $aParams['publisher_id'] = $this->pageParams['affiliateid'];
            
            $zones = Admin_DA::getZones($aParams, false);
            foreach ($zones as $zone) {
                phpAds_PageContext (
                    phpAds_buildName($zone['zone_id'], $zone['name']),
                    $this->pageName.'?affiliateid='.$aParams['publisher_id'].'&zoneid='.$zone['zone_id'],
                    $current_id == $zone['zone_id']
                );
            }
            break;
        
        }
    }
    
    /**
     * Add breadcrumbs for an entity, automatically adding parent entities if needed
     *
     * @param string Entity type (advertiser, campaign, etc)
     * @param integer Entity ID
     * @param intever Recursion level
     */
    function addBreadcrumbs($type, $entityId, $level = 0)
    {
        switch ($type) {
        
        case 'advertiser':
            if (phpAds_isUser(phpAds_Admin) || phpAds_isUser(phpAds_Agency)) {
                $advertisers = Admin_DA::getAdvertisers(array('advertiser_id' => $entityId), false);
                if (count($advertisers) == 1) {
                    $advertiser = current($advertisers);
                    $this->_addBreadcrumb(
                        phpAds_buildName($advertiser['advertiser_id'], $advertiser['name']),
                        MAX_getEntityIcon('advertiser')
                    );
                }
            }
            
            break;
            
        case 'campaign':
            $campaigns = Admin_DA::getPlacements(array('placement_id' => $entityId), false);
            if (count($campaigns) == 1) {
                $campaign = current($campaigns);
                $this->_addBreadcrumb(
                    phpAds_buildName($campaign['placement_id'], $campaign['name']),
                    MAX_getEntityIcon('placement')
                );
                
                $this->addBreadcrumbs('advertiser', $campaign['advertiser_id'], $level + 1);
            }
            
            break;
            
        case 'banner':
            $banners = Admin_DA::getAds(array('ad_id' => $entityId), false);
            if (count($banners) == 1) {
                $banner = current($banners);
                $this->_addBreadcrumb(
                    phpAds_buildName($banner['banner_id'], $banner['name']),
                    MAX_getEntityIcon('ad')
                );
                
                $this->addBreadcrumbs('campaign', $banner['placement_id'], $level + 1);
            }
            
            break;
            
        case 'publisher':
            if (phpAds_isUser(phpAds_Admin) || phpAds_isUser(phpAds_Agency)) {
                $publishers = Admin_DA::getPublishers(array('publisher_id' => $entityId), false);
                if (count($publishers) == 1) {
                    $publisher = current($publishers);
                    $this->_addBreadcrumb(
                        phpAds_buildName($publisher['publisher_id'], $publisher['name']),
                        MAX_getEntityIcon('publisher')
                    );
                }
            }
            
            break;
            
        case 'zone':
            $zones = Admin_DA::getZones(array('zone_id' => $entityId), false);
            if (count($zones) == 1) {
                $zone = current($zones);
                $this->_addBreadcrumb(
                    phpAds_buildName($zone['zone_id'], $zone['name']),
                    MAX_getEntityIcon('zone')
                );
                
                $this->addBreadcrumbs('publisher', $zone['publisher_id'], $level + 1);
            }
            
            break;
        
        }
        
        if (!$level) {
            $this->pageBreadcrumbs = array_reverse($this->pageBreadcrumbs);
        }
    }
    
    /**
     * Internal function to manually add a breadcrumb
     *
     * @param string Breadcrumb text
     * @param string Breadcrumb icon
     */
    function _addBreadcrumb($name, $icon)
    {
        $this->pageBreadcrumbs[] = array('name' => $name, 'icon' => $icon);
    }
    
    /**
     * Output the breadcrumbs, highlighing the last item
     */
    function showBreadcrumbs()
    {
        if (is_array($this->pageBreadcrumbs)) {
            foreach ($this->pageBreadcrumbs as $k => $bc) {
                if ($k == count($this->pageBreadcrumbs) - 1) {
                    $bc['name'] = '<b>'.$bc['name'].'</b>';
                }
                if ($k > 0) {
                    echo "&nbsp;<img src='images/".$GLOBALS['phpAds_TextDirection']."/caret-rs.gif'>&nbsp;";
                }
                echo '<img src="'.$bc['icon'].'" align="absmiddle" />&nbsp;'.$bc['name'];
            }
            
            if (count($this->pageBreadcrumbs)) {
                echo "<br /><br /><br />";
            }
        }
    }

    /**
     * Add a shortuct in the left navigation bar
     *
     * @param string Shortuct text
     * @param string Shortcut link
     * @param string Shortcut icon
     */
    function addShortcut($name, $link, $icon)
    {
        $this->pageShortcuts[] = array('name' => $name, 'link' => $link, 'icon' => $icon);
    }
    
    /**
     * Output the shortcuts
     *
     * Note: the function uses phpAds_PageShortcut(), it doesn't make any output itself
     */
    function showShortcuts()
    {
        foreach ($this->pageShortcuts as $shortcut) {
            phpAds_PageShortcut($shortcut['name'], $shortcut['link'], $shortcut['icon']);
        }
    }
    
    /**
     * Output the error string when stats are not available
     *
     * @return string Error string
     */
    function showNoStatsString()
    {
        if (!empty($this->aDates['day_begin']) && !empty($this->aDates['day_end'])) {
            $startDate = & new Date($this->aDates['day_begin']);
            $startDate = $startDate->format($GLOBALS['date_format']);
            $endDate   = & new Date($this->aDates['day_end']);
            $endDate   = $endDate->format($GLOBALS['date_format']);
            return sprintf($GLOBALS['strNoStatsForPeriod'], $startDate, $endDate);
        }
        
        return $GLOBALS['strNoStats'];
    }

    /**
     * Output the controller object using a template
     */
    function output($elements = array()) {
        $output = new HTML_Template_Flexy(array(
            'templateDir'       => MAX_PATH . '/lib/max/Admin/Statistics/themes',
            'compileDir'        => MAX_PATH . '/var/templates_compiled',
        ));
        
        // Add global variables for backwards compatibility
        if (phpAds_isUser(phpAds_Client)) {
            $GLOBALS['clientid'] = phpAds_getUserId();
        } elseif (phpAds_isUser(phpAds_Affiliate)) {
            $GLOBALS['affiliateid'] = phpAds_getUserId();
        }
        
        // Generate URI used to add other parameters
        $this->generatePageURI();
        
        // Add context links, if any
        if (is_array($this->pageContext))
            call_user_func_array(array($this, 'showContext'), $this->pageContext);

        // Add shortcuts, if any
        $this->showShortcuts();
        
        // Display header and section links
        phpAds_PageHeader($this->pageId);
        $this->showBreadcrumbs();
        phpAds_ShowSections($this->pageSections);

        // Display page content
        $output->compile($this->template);
        $output->outputObject($this, $elements);
        
        $this->savePrefs();
        phpAds_PageFooter();
    }
}

?>
