<?php

/*
+---------------------------------------------------------------------------+
| Max Media Manager v0.3                                                    |
| =================                                                         |
|                                                                           |
| Copyright (c) 2003-2006 m3 Media Services Limited                         |
| For contact details, see: http://www.m3.net/                              |
|                                                                           |
| This program is free software; you can redistribute it and/or modify      |
| it under the terms of the GNU General Public License as published by      |
| the Free Software Foundation; either version 2 of the License, or         |
| (at your option) any later version.                                       |
|                                                                           |
| This program is distributed in the hope that it will be useful,           |
| but WITHOUT ANY WARRANTY; without even the implied warranty of            |
| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
| GNU General Public License for more details.                              |
|                                                                           |
| You should have received a copy of the GNU General Public License         |
| along with this program; if not, write to the Free Software               |
| Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA |
+---------------------------------------------------------------------------+
$Id: StatsHistoryController.php 4643 2006-04-10 14:19:05Z matteo@beccati.com $
*/

require_once MAX_PATH . '/lib/max/Admin/Statistics/StatsController.php';
require_once 'Pager/Pager.php';



/**
 * Controller class for displaying history type statistics screens
 *
 * Always use the factory method to instantiate fields -- it will create
 * the right subclass for you.
 *
 * @package    Max
 * @subpackage Admin_Statistics
 * @author     Matteo Beccati <matteo@beccati.com>
 *
 * @see StatsControllerFactory
 */
class StatsHistoryController extends StatsController
{
    var $history;
    
    var $startDate;
    var $spanDays;
    var $spanWeeks;
    var $spanMonths;
    
    var $statsBreakdown;
    var $statsKey;
    var $statsIcon;
    
    var $grandTotal;
    var $grandAverage;
    var $grandAverageSpan;

    var $averageDesc;
    
    /**
     * PHP5-style constructor
     */
    function __constructor()
    {
        // Get list order and direction
        $this->listOrderField     = MAX_getStoredValue('listorder', 'key');
        $this->listOrderDirection = MAX_getStoredValue('orderdirection', 'down');
        $this->statsBreakdown     = MAX_getStoredValue('breakdown', 'day');

        parent::__constructor();

        // Store the preferences
        $this->pagePrefs['listorder']      = $this->listOrderField;
        $this->pagePrefs['orderdirection'] = $this->listOrderDirection;
        $this->pagePrefs['breakdown']      = $this->statsBreakdown;
    }

    /**
     * PHP4-style constructor
     */
    function StatsHistoryController()
    {
        $this->__constructor();
    }
    
    /**
     * Output the controller object using the breakdown_by_date template
     *
     * @param boolean Show the breakdown selector
     */
    function output($show_breakdown = true)
    {
        $this->template = 'breakdown_by_date.html';

        $elements = array();
        if ($show_breakdown) {
            $elements['breakdown'] = new HTML_Template_Flexy_Element;
            $elements['breakdown']->setOptions( array(
              'day'   => $GLOBALS['strBreakdownByDay'],
              'month' => $GLOBALS['strBreakdownByMonth'],
              'dow'   => $GLOBALS['strBreakdownByDow'],
              'hour'  => $GLOBALS['strBreakdownByHour']
            ));
            $elements['breakdown']->setValue($this->statsBreakdown);
            $elements['breakdown']->setAttributes(array('onchange' => 'this.form.submit()'));
        }
        
        parent::output($elements);
    }
    
    /**
     * Fetch the available span for the stats using the specified parameters
     *
     * @param array Query parameters
     */
    function getHistorySpan($aParams)
    {
        global $conf;
        
        $span = Admin_DA::fromCache('getHistorySpan', $aParams);
        
        if (!empty($span['start_date'])) {
            $start_date = & new Date($span['start_date']);
        } else {
            $start_date = & new Date(date('Y-m-d'));
        }

        $now  = & new Date();       
        $span = & new Date_Span($start_date, new Date());
        
        $this->startDate  = $start_date;
        $this->spanDays   = (int)ceil($span->toDays());
        $this->spanWeeks  = (int)ceil($span_days / 7) + ($span_days % 7 ? 1 : 0);
        $this->spanMonths = (($now->getYear() - $start_date->getYear()) * 12) + ($now->getMonth() - $start_date->getMonth()) + 1;
    }

    /**
     * Fetch and decorates the history stats using the specified parameters
     *
     * @param array Query parameters
     * @param string Optional link for the leftmost column content
     */
    function prepareHistory($aParams, $link = '')
    {
        $this->getHistorySpan($aParams);
        
        $stats = $this->getHistory($aParams, $link);
        
        if ($this->noStatsAvailable) {
            $this->history = array();
            return;
        }

        if ($this->statsBreakdown == 'hour') {
            $this->statsIcon = 'images/icon-time.gif';
        } else {
            $this->statsIcon = 'images/icon-date.gif';
        }
        
        if ($this->globalPrefs['period_preset'] == 'this_month' || $this->globalPrefs['period_preset'] == 'last_month') {
            $use_pager = false;
        } else {
            $use_pager = count($stats) > 30;
        }

        if ($use_pager) {
            $params = array(
                'itemData' => $stats,
                'perPage' => MAX_getStoredValue('setPerPage', 30),
                'delta' => 8,
                'append' => true,
                'clearIfVoid' => false,
                'urlVar' => 'page',
                'useSessions' => false,
                'mode'  => 'Jumping'
            );
            
            $pager = & Pager::factory($params);
            $this->history = $pager->getPageData();
            $this->pagerLinks = $pager->getLinks();
            $this->pagerLinks = $this->pagerLinks['all'];
            
            $this->pagerSelect = preg_replace('/(<select.*?)(>)/i', '$1 onchange="this.form.submit()"$2',
                $pager->getPerPageSelectBox(15, 60, 15));
        } else {
            $this->history = $stats;
            $this->pagerLinks = false;
            $this->pagerSelect = false;
        }
                
        $this->pagePrefs['setPerPage'] = $params['perPage'];

        if (count($this->history)) {
            $i = 0;
            foreach(array_keys($this->history) as $k) {
                $this->history[$k]['htmlclass'] = ($i++ % 2 == 0) ? 'dark' : 'light';
                
                if ($i == count($this->history)) {
                    $this->history[$k]['htmlclass'] .= ' last';
                }
            }
        }
    }
    
    /**
     * Fetch the history stats using the specified parameters
     *
     * @param array Query parameters
     * @param string Optional link for the leftmost column content
     */
    function getHistory($aParams, $link = '')
    {
        switch ($this->statsBreakdown) {
        case 'month':
            $this->statsKey = $GLOBALS['strSingleMonth'];
            $this->averageDesc = $GLOBALS['strMonths'];
            $method = 'getMonthHistory';
            break;
            
        case 'dow'  :
            $this->statsKey = $GLOBALS['strDayOfWeek'];
            $this->averageDesc = $GLOBALS['strWeekDays'];
            $method = 'getDayOfWeekHistory';
            break;
            
        case 'hour' :
            $this->statsKey = $GLOBALS['strHour'];
            $this->averageDesc = $GLOBALS['strHours'];
            $method = 'getHourHistory';
            break;
            
        default     :
            $this->statsBreakdown = 'day';
            $this->statsKey = $GLOBALS['strDay'];
            $this->averageDesc = $GLOBALS['strDays'];
            $method = 'getDayHistory';
            break;
        }
        
        
        $stats = Admin_DA::fromCache($method, $aParams + $this->aDates);
        
        if (count($stats)) {
            $this->fillGaps($stats, $link);
                    
            if (!in_array($this->listOrderField, array($this->statsBreakdown, 'sum_requests', 'sum_views', 'sum_clicks', 'sum_ctr', 'sum_conversions', 'sum_sr'))) {
                $this->listOrderField = $this->statsBreakdown;
                $this->listOrderDirection = $this->statsBreakdown == 'hour' || $this->statsBreakdown == 'dow' ? 'up' : 'down';
            }
            
            MAX_sortArray($stats, $this->listOrderField, $this->listOrderDirection == 'up');
            
            $this->summarizeTotals($stats, true);
        } else {
            $this->noStatsAvailable = true;
        }
        
        return $stats;
    }

    /**
     * Return an array which contains all the interval days, months, etc
     *
     * The key is uses the internal format for the date, and the value is its
     * formatted counterpart
     *
     * @return array Dates array
     */
    function getDatesArray()
    {
        return $this->_getDatesArray($this->statsBreakdown, $this->aDates, $this->startDate);
    }
    
    /**
     * Internal static function which returns an array which contains all the interval days, months, etc
     *
     * The key is uses the internal format for the date, and the value is its
     * formatted counterpart
     *
     * @param string Breakdown type
     * @param array Dates array
     * @param object Date object representing the first day of statistics available
     *
     * @return array Dates array
     */
    function _getDatesArray($breakdown, $aDates, $startDate)
    {
        // Get start and end dates
        if ($aDates['day_begin'] && $aDates['day_end']) {
            $start_date = & new Date($aDates['day_begin']);
            $end_date   = & new Date($aDates['day_end']);
            
            // Adjust start date if no stats are available
            if ($start_date->before($startDate)) {
                $aDates['day_begin'] = $startDate->format('%Y-%m-%d');
                $start_date = & new Date($aDates['day_begin']);
            }
            
            // Adjust end date if is future
            if ($end_date->isFuture()) {
                $aDates['day_end'] = date('Y-m-d');
                $end_date = & new Date($aDates['day_end']);
            }
        } else {
            // Use all available stats
            $start_date = $startDate;
            $end_date   = & new Date(date('Y-m-d'));
        }
        
        $dates = array();
        switch ($breakdown) {
        
        case 'day' :
            $one_day = &new Date_Span('1', '%d');
            $end_date->addSpan($one_day);
            $date = &new Date($start_date);
            while ($date->before($end_date)) {
                $dates[$date->format('%Y-%m-%d')] = $date->format($GLOBALS['date_format']);    
                $date->addSpan($one_day);
            }
            break;
        
        case 'month' :
            $one_month = &new Date_Span((string)($end_date->getDaysInMonth() - $end_date->getDay() + 1), '%d');
            $end_date->addSpan($one_month);
            $date = &new Date($start_date);
            while ($date->before($end_date)) {
                $dates[$date->format('%Y-%m')] = $date->format($GLOBALS['month_format']);
                $one_month = &new Date_Span((string)($date->getDaysInMonth() - $date->getDay() + 1), '%d');
                $date->addSpan($one_month);
            }
            break;
        
        case 'dow' :
            for ($dow = 0; $dow < 7; $dow++) {
                $dates[$dow] = $GLOBALS['strDayFullNames'][$dow];
            }
            break;
        
        case 'hour' :
            for ($hour = 0; $hour < 24; $hour++) {
                $dates[$hour] = sprintf('%02d:00 - %02d:59', $hour, $hour);
            }
            break;

        }

        return $dates;
    }
    
    /**
     * Fills the gaps in the history stats and adds required links
     *
     * @param array Stats array
     */
    function fillGaps(&$stats, $link)
    {
        foreach ($this->getDatesArray() as $key  => $date_f) {
            if (!isset($stats[$key])) {
                $stats[$key] = array(
                    $this->statsBreakdown => $key,
                    'sum_requests' => 0,
                    'sum_views' => 0,
                    'sum_clicks' => 0,
                    'sum_ctr' => 0,
                    'sum_conversions' => 0,
                    'sum_sr' => 0
                );
            }
            $stats[$key]['date_f'] = $date_f;
            $this->summarizeStats($stats[$key]);
                
            switch ($this->statsBreakdown) {
            
            case 'day' :
                $stats[$key]['link'] = $this->uriAddParams($link).'day='.str_replace('-', '', $key);
                $stats[$key]['conversionslink'] = $this->uriAddParams('stats-conversions.php').
                    'day='.str_replace('-', '', $key);
                break;
            
            case 'month' :
                $month_start = & new Date(sprintf('%s-%02d', $key, 1));
                $month_end = & new Date($month_start);
                $month_end->setDay($month_end->getDaysInMonth());
                $stats[$key]['conversionslink'] = $this->uriAddParams('stats-conversions.php').
                    'period_preset=specific&'.
                    'period_start='.$month_start->format('%Y-%m-%d').'&'.
                    'period_end='.$month_end->format('%Y-%m-%d');
                break;
            
            case 'dow' :
                // do nothing
                break;
            
            case 'hour' :
                if (!empty($this->aDates['day_begin']) && $this->aDates['day_begin'] == $this->aDates['day_end']) {
                    $stats[$key]['conversionslink'] = $this->uriAddParams('stats-conversions.php').
                        'day='.str_replace('-', '', $this->aDates['day_begin']).'&'.
                        'hour='.$key;
                }
                break;
            }
        }
    }
}

?>
