<?php

/*
+---------------------------------------------------------------------------+
| Max Media Manager v0.3                                                    |
| =================                                                         |
|                                                                           |
| Copyright (c) 2003-2006 m3 Media Services Limited                         |
| For contact details, see: http://www.m3.net/                              |
|                                                                           |
| This program is free software; you can redistribute it and/or modify      |
| it under the terms of the GNU General Public License as published by      |
| the Free Software Foundation; either version 2 of the License, or         |
| (at your option) any later version.                                       |
|                                                                           |
| This program is distributed in the hope that it will be useful,           |
| but WITHOUT ANY WARRANTY; without even the implied warranty of            |
| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
| GNU General Public License for more details.                              |
|                                                                           |
| You should have received a copy of the GNU General Public License         |
| along with this program; if not, write to the Free Software               |
| Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA |
+---------------------------------------------------------------------------+
$Id: AllocateZoneImpressions.php 4346 2006-03-06 16:43:19Z andrew@m3.net $
*/

require_once MAX_PATH . '/lib/max/Maintenance/Statistics/Common/Task.php';

/**
 * A class for summarising raw data into the intermediate tables, for the
 * AdServer module.
 *
 * @package    MaxMaintenance
 * @subpackage Statistics
 * @author     Andrew Hill <andrew@m3.net>
 */
class MAX_Maintenance_Statistics_AdServer_Task_SummariseIntermediate extends MAX_Maintenance_Statistics_Common_Task
{

    /**
     * The constructor method.
     *
     * @return MAX_Maintenance_Statistics_AdServer_Task_SummariseIntermediate
     */
    function MAX_Maintenance_Statistics_AdServer_Task_SummariseIntermediate()
    {
        parent::MAX_Maintenance_Statistics_Common_Task();
    }

    /**
     * The implementation of the MAX_Core_Task::run() method that performs
     * the task of this class.
     */
    function run()
    {
        $conf = $GLOBALS['_MAX']['CONF'];
        if (($this->oController->updateFinal) && (!$this->oController->updateUsingOI)) {
            // Summarise the intermediate stats by the hour
            $oStartDate = new Date();
            $oStartDate->copy($this->oController->lastDateFinal);
            $oStartDate->addSeconds(1);
            while (Date::compare($oStartDate, $this->oController->updateFinalToDate) < 0) {
                // Calculate the end of the hour
                $oEndDate = new Date();
                $oEndDate->copy($oStartDate);
                $oEndDate->addSeconds(SECONDS_PER_HOUR - 1); // Plus one hour
                // Summarise the requests
                $this->_summariseIntermediateRequests($oStartDate, $oEndDate);
                // Summarise the impressions
                $this->_summariseIntermediateImpressions($oStartDate, $oEndDate);
                // Summarise the clicks
                $this->_summariseIntermediateClicks($oStartDate, $oEndDate);
                if ($conf['modules']['Tracker']) {
                    // Summarise the connections
                    $this->_summariseIntermediateConnections($oStartDate, $oEndDate);
                }
                // Save the impressions/clicks/connections
                $this->_saveIntermediateSummaries($oStartDate, $oEndDate);
                // Go to the next hour
                $oStartDate->addSeconds(3600);
            }
        } elseif (($this->oController->updateIntermediate) && ($this->oController->updateUsingOI)) {
            // Summarise the intermediate stats by the operation interval
            $oStartDate = new Date();
            $oStartDate->copy($this->oController->lastDateIntermediate);
            $oStartDate->addSeconds(1);
            while (Date::compare($oStartDate, $this->oController->updateIntermediateToDate) < 0) {
                // Calcuate the end of the operation interval
                $oEndDate = new Date();
                $oEndDate->copy($oStartDate);
                $oEndDate->addSeconds(($conf['maintenance']['operationInterval'] * 60) - 1); // Plus one OI
                // Summarise the requests
                $this->_summariseIntermediateRequests($oStartDate, $oEndDate);
                // Summarise the impressions
                $this->_summariseIntermediateImpressions($oStartDate, $oEndDate);
                // Summarise the clicks
                $this->_summariseIntermediateClicks($oStartDate, $oEndDate);
                if ($conf['modules']['Tracker']) {
                    // Summarise the connections
                    $this->_summariseIntermediateConnections($oStartDate, $oEndDate);
                }
                // Save the impressions/clicks/connections
                $this->_saveIntermediateSummaries($oStartDate, $oEndDate);
                // Go to the next operation interval
                $oStartDate->addSeconds($conf['maintenance']['operationInterval'] * 60);
            }
        }
    }

    /**
     * A private method to summarise the ad requests, ready for insertion into
     * intermediate statistics tables.
     *
     * @access private
     * @param PEAR::Date $oStartDate The start date of the data to summarise.
     * @param PEAR::Date $oEndDate The end date of the data to summarise.
     */
    function _summariseIntermediateRequests($oStartDate, $oEndDate)
    {
        $oServiceLocator = &ServiceLocator::instance();
        $oDal = &$oServiceLocator->get('MAX_Dal_Statistics_AdServer');
        $startTime = time();
        $this->oController->report .= "Summarising requests for the intermediate tables.\n";
        MAX::debug('Summarising requests for the intermediate tables', PEAR_LOG_DEBUG);
        $this->oController->report .= 'Summarising requests between ' . $oStartDate->format('%Y-%m-%d %H:%M:%S') . ' and ' .
                                      $oEndDate->format('%Y-%m-%d %H:%M:%S') . ".\n";
        MAX::debug('Summarising requests between ' . $oStartDate->format('%Y-%m-%d %H:%M:%S') . ' and ' .
                   $oEndDate->format('%Y-%m-%d %H:%M:%S'), PEAR_LOG_DEBUG);
        $rows = $oDal->summariseRequests($oStartDate, $oEndDate);
        $runTime = time() - $startTime;
        $this->oController->report .= "Summarised $rows request rows in $runTime seconds.\n";
        MAX::debug("Summarised $rows request rows in $runTime seconds", PEAR_LOG_DEBUG);
    }

    /**
     * A private method to summarise the ad impressions, ready for insertion into
     * intermediate statistics tables.
     *
     * @access private
     * @param PEAR::Date $oStartDate The start date of the data to summarise.
     * @param PEAR::Date $oEndDate The end date of the data to summarise.
     */
    function _summariseIntermediateImpressions($oStartDate, $oEndDate)
    {
        $oServiceLocator = &ServiceLocator::instance();
        $oDal = &$oServiceLocator->get('MAX_Dal_Statistics_AdServer');
        $startTime = time();
        $this->oController->report .= "Summarising impressions for the intermediate tables.\n";
        MAX::debug('Summarising impressions for the intermediate tables', PEAR_LOG_DEBUG);
        $this->oController->report .= 'Summarising impressions between ' . $oStartDate->format('%Y-%m-%d %H:%M:%S') . ' and ' .
                                      $oEndDate->format('%Y-%m-%d %H:%M:%S') . ".\n";
        MAX::debug('Summarising impressions between ' . $oStartDate->format('%Y-%m-%d %H:%M:%S') . ' and ' .
                   $oEndDate->format('%Y-%m-%d %H:%M:%S'), PEAR_LOG_DEBUG);

        $rows = $oDal->summariseImpressions($oStartDate, $oEndDate);
        $runTime = time() - $startTime;
        $this->oController->report .= "Summarised $rows impression rows in $runTime seconds.\n";
        MAX::debug("Summarised $rows impression rows in $runTime seconds", PEAR_LOG_DEBUG);
    }

    /**
     * A private method to summarise the ad clicks, ready for insertion into
     * intermediate statistics tables.
     *
     * @access private
     * @param PEAR::Date $oStartDate The start date of the data to summarise.
     * @param PEAR::Date $oEndDate The end date of the data to summarise.
     */
    function _summariseIntermediateClicks($oStartDate, $oEndDate)
    {
        $oServiceLocator = &ServiceLocator::instance();
        $oDal = &$oServiceLocator->get('MAX_Dal_Statistics_AdServer');
        $startTime = time();
        $this->oController->report .= "Summarising clicks for the intermediate tables.\n";
        MAX::debug('Summarising clicks for the intermediate tables', PEAR_LOG_DEBUG);
        $this->oController->report .= 'Summarising clicks between ' . $oStartDate->format('%Y-%m-%d %H:%M:%S') . ' and ' .
                                      $oEndDate->format('%Y-%m-%d %H:%M:%S') . ".\n";
        MAX::debug('Summarising clicks between ' . $oStartDate->format('%Y-%m-%d %H:%M:%S') . ' and ' .
                   $oEndDate->format('%Y-%m-%d %H:%M:%S'), PEAR_LOG_DEBUG);
        $rows = $oDal->summariseClicks($oStartDate, $oEndDate);
        $runTime = time() - $startTime;
        $this->oController->report .= "Summarised $rows click rows in $runTime seconds.\n";
        MAX::debug("Summarised $rows click rows in $runTime seconds", PEAR_LOG_DEBUG);
    }

    /**
     * A private method to summarise the ad connections, ready for insertion into
     * intermediate statistics tables.
     *
     * @access private
     * @param PEAR::Date $oStartDate The start date of the data to summarise.
     * @param PEAR::Date $oEndDate The end date of the data to summarise.
     */
    function _summariseIntermediateConnections($oStartDate, $oEndDate)
    {
        $oServiceLocator = &ServiceLocator::instance();
        $oDal = &$oServiceLocator->get('MAX_Dal_Statistics_AdServer');
        $startTime = time();
        $this->oController->report .= "Summarising connections for the intermediate tables.\n";
        MAX::debug('Summarising connections for the intermediate tables', PEAR_LOG_DEBUG);
        $this->oController->report .= 'Summarising connections between ' . $oStartDate->format('%Y-%m-%d %H:%M:%S') . ' and ' .
                                       $oEndDate->format('%Y-%m-%d %H:%M:%S') . ".\n";
        MAX::debug('Summarising connections between ' . $oStartDate->format('%Y-%m-%d %H:%M:%S') . ' and ' .
                   $oEndDate->format('%Y-%m-%d %H:%M:%S'), PEAR_LOG_DEBUG);
        $rows = $oDal->summariseConnections($oStartDate, $oEndDate);
        $runTime = time() - $startTime;
        $this->oController->report .= "Summarised $rows connection rows in $runTime seconds.\n";
        MAX::debug("Summarised $rows connection rows in $runTime seconds", PEAR_LOG_DEBUG);
    }

    /**
     * A private method to save the summarised request/impressions/clicks/connections
     * into the intermediate tables.
     *
     * @access private
     * @param PEAR::Date $oStartDate The start date of the data to save.
     * @param PEAR::Date $oEndDate The end date of the data to save.
     */
    function _saveIntermediateSummaries($oStartDate, $oEndDate)
    {
        $oServiceLocator = &ServiceLocator::instance();
        $oDal = &$oServiceLocator->get('MAX_Dal_Statistics_AdServer');
        $conf = $GLOBALS['_MAX']['CONF'];
        if ($conf['modules']['Tracker']) {
            $this->oController->report .= "Saving request, impression, click and connection data into the intermediate tables.\n";
            MAX::debug('Saving request, impression, click and connection data into the intermediate tables', PEAR_LOG_DEBUG);
        } else {
            $this->oController->report .= "Saving request, impression, click data into the intermediate tables.\n";
            MAX::debug('Saving request, impression, click data into the intermediate tables', PEAR_LOG_DEBUG);
        }
        $oDal->saveIntermediate($oStartDate, $oEndDate);
    }

}

?>
